#include <stdlib.h>
#include <stdio.h>
#include <mem.h>
#include <string.h>

#include "types.h"
#include "magic.h"
#include "md5.h"
#include "chap.h"

void MD5InitO(MD5_CTX *);
void MD5UpdateO(MD5_CTX *, unsigned char *, unsigned int);
void MD5FinalO(MD5_CTX *);

static int myrand(int num)
{
    return (int) ( ((long)rand() * (num)) / (RAND_MAX + 1) );
}

static void ChapGenChallenge(chap_state *cstate)
{
    int chal_len;
    u_char *ptr = cstate->challenge;
    unsigned int i;

    /* pick a random challenge length between MIN_CHALLENGE_LENGTH and
       MAX_CHALLENGE_LENGTH */
    chal_len = (unsigned)(myrand(MAX_CHALLENGE_LENGTH - MIN_CHALLENGE_LENGTH)
			  + MIN_CHALLENGE_LENGTH);
    cstate->chal_len = chal_len;
    cstate->chal_id = ++cstate->id;
    cstate->chal_transmits = 0;

    /* generate a random string */
    for (i = 0; i < chal_len; i++ )
	*ptr++ = (char)(myrand(0x00ff) + 1);
}

int main(void)
{
    int secret_len, i;
    MD5_CTX mdContext, mdContextO;
    char secret[256];

    chap_state *cstate;

    magic_init();

    if ( (cstate = calloc(1, sizeof(chap_state))) == NULL )
	return -1;

    for ( i = 0 ; i < 32000 ; ++i ) {
	ChapGenChallenge(cstate);
	strcpy(secret, "Esto es un secreto, sssss...");
	secret_len = strlen(secret);

	MD5Init(&mdContext);
	MD5Update(&mdContext, &cstate->resp_id, 1);
	MD5Update(&mdContext, secret, secret_len);
	MD5Update(&mdContext, cstate->challenge, cstate->chal_len);
	MD5Final(&mdContext);
	memcpy(cstate->response, mdContext.digest, MD5_SIGNATURE_SIZE);
	cstate->resp_length = MD5_SIGNATURE_SIZE;

	MD5InitO(&mdContextO);
	MD5UpdateO(&mdContextO, &cstate->resp_id, 1);
	MD5UpdateO(&mdContextO, secret, secret_len);
	MD5UpdateO(&mdContextO, cstate->challenge, cstate->chal_len);
	MD5FinalO(&mdContextO);

	if ( memcmp(cstate->response, mdContextO.digest, MD5_SIGNATURE_SIZE) ) {
	    printf("No son iguales\n");
	    break;
	}
	else
	    fprintf(stderr, "\rpaso: %d", i);
    }

    return 0;
}

