/*
 * Small program for displaying VJ compression statitistics for Dospppd.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>
#include <mem.h>
#include "types.h"
#include "asyimprt.h"
#include "pktdrvr.h"
#include "ip.h"
#include "tcp.h"
#include "global.h"
#include "slhc_vj.h"

#undef  DEBUGARP 1

/****************** ARP stuff ********************/

typedef struct {                /* ARP packet full format */
    u_char    ethdestin[6];     /* Ethernet header */
    u_char    ethsource[6];
    u_short   ethproto;

    u_short   hw;               /* Hardware address length, bytes */
    u_short   prot;             /* Protocol type */
    u_char    hwalen;           /* hardware address length, bytes */
    u_char    pralen;           /* Length of protocol address */
    u_short   opcode;           /* ARP opcode (request/reply) */
    u_char    shwaddr[6];       /* Sender hardware address field */
    u_int32_t sprotaddr;        /* Sender Protocol address field */
    u_char    thwaddr[6];       /* Target hardware address field */
    u_int32_t tprotaddr;        /* Target protocol address field */
} ARP;
typedef ARP far * ARP_FP;

#ifdef  DEBUGARP
typedef struct {                /* ARP statistics, mainly for debugging */
  u_int32_t requests;
  u_int32_t localsearch;
  u_int32_t noreceiver;
  u_int32_t nobuffer;
  u_int32_t replies;
} ARPSTATS;
typedef ARPSTATS far * ARPSTATS_FP;
#endif  /* DEBUGARP */

ASY_HOOKS asyfuncs = {
    sizeof(asyfuncs)
};

int find_packet_driver(int);

int main(int argc, char *argv[])
{
    int pktint = 0;
    char *ptr;
    u_int32_t res;
    union REGS regs;
    struct SREGS sregs;
    struct slcompress vjstats, far *vjstatsfp;
#ifdef  DEBUGARP
    ARPSTATS arpstats;
    ARPSTATS_FP arpstatsfp = NULL;
#endif  /* DEBUGARP */

    /* get user specified packet vector if any */
    if ( argc > 1 ) {
        if ( argc > 2 ) {
            fprintf(stderr, "Usage: VJCSTAT [vecnum]\n");
            return 1;
        }

        res = strtoul(argv[1], &ptr, 0);

        if ( ptr == *argv || *ptr != 0 || res == 0 || res > 255 ) {
            fprintf(stderr, "Syntax error, invalid vecnum\n");
            return 2;
        }

        pktint = (int)res;
    }

    /* look for a installed copy of PPPD */
    if ( (pktint = find_packet_driver(pktint)) == 0 ) {
        fprintf(stderr, "PPPD packet driver not found");
        return 3;
    }

    /* request VJ comp stats */
    segread(&sregs);
    regs.x.bx = sizeof(struct slcompress);
#ifdef  DEBUGARP
    /* trick for getting ARP statistics pointer */
    regs.x.cx = sizeof(ARPSTATS);
#endif  /* DEBUGARP */
    regs.h.ah = GET_VJSTATS;
    int86x(pktint, &regs, &regs, &sregs);

    /* if CY is set, we fail */
    if ( regs.x.cflag ) {
        fprintf(stderr, "GET_VJSTATS isn't supported by this PD !");
        return 4;
    }

    /* get a copy of the structure */
    vjstatsfp = (struct slcompress far *)MK_FP(sregs.ds, regs.x.si);
    disable();
    _fmemcpy((void far *)&vjstats, vjstatsfp, sizeof(struct slcompress));
    enable();
#ifdef  DEBUGARP
    /* get a copy of ARP statistics if driver returns a valid pointer (?) */
    if ( regs.x.bx != sizeof(struct slcompress) ) {
        arpstatsfp = (ARPSTATS_FP)MK_FP(sregs.ds, regs.x.bx);
        _fmemcpy((ARPSTATS_FP)&arpstats, arpstatsfp, sizeof(ARPSTATS));
    }
#endif  /* DEBUGARP */
    /* dump information to stdout */
    printf("highest transmit slot: %-15u", vjstats.tslot_limit);
    printf("highest receive slot: %u\n", vjstats.rslot_limit);
    printf("most recent xmit id: %-17u", vjstats.xmit_current);
    printf("most recent rcvd id: %u\n", vjstats.recv_current);
    printf("connection state searches: %-11lu", vjstats.sls_o_searches);
    printf("connection state misses: %lu\n", vjstats.sls_o_misses);
    printf("oldest xmit in ring: %-17u", vjstats.xmit_oldest);
    printf("compression state flags: 0x%x\n", vjstats.flags);
    printf("out non-TCP packets: %-17lu", vjstats.sls_o_nontcp);
    printf("out TCP packets: %lu\n", vjstats.sls_o_tcp);
    printf("out uncompressed packets: %-12lu", vjstats.sls_o_uncompressed);
    printf("out compressed packets: %lu\n", vjstats.sls_o_compressed);
    printf("in uncompressed packets: %-13lu", vjstats.sls_i_uncompressed);
    printf("in compressed packets: %lu\n", vjstats.sls_i_compressed);
    printf("in error packets: %-20lu", vjstats.sls_i_error);
    printf("in error packets tossed: %lu\n", vjstats.sls_i_tossed);
    printf("in runt packets: %-21lu", vjstats.sls_i_runt);
    printf("in packets with bad checksum: %lu\n", vjstats.sls_i_badcheck);
#ifdef  DEBUGARP
    /* dump ARP info if it was found */
    if ( arpstatsfp ) {
        ARP_FP rcv_arp_req;
        ARP_FP snd_arp_rpl;
        ARP req, rpl;
        u_short cnt;

        printf("\nARP requests: %lu\n", arpstats.requests);
        printf("ARP local searches: %lu\n", arpstats.localsearch);
        printf("ARP fail no receiver: %lu\n", arpstats.noreceiver);
        printf("ARP fail no buffer: %lu\n", arpstats.nobuffer);
        printf("ARP replies: %lu\n\n", arpstats.replies);

        rcv_arp_req = (ARP_FP)(arpstatsfp + 1);
        snd_arp_rpl = (ARP_FP)(rcv_arp_req + 16);

        for ( cnt = 0 ; cnt < 16 ; ++cnt, ++rcv_arp_req, ++snd_arp_rpl ) {
            _fmemcpy(&req, rcv_arp_req, sizeof(ARP));
            _fmemcpy(&rpl, snd_arp_rpl, sizeof(ARP));

            printf("req.ethdestin = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X ",
                   req.ethdestin[0], req.ethdestin[1],
                   req.ethdestin[2], req.ethdestin[3],
                   req.ethdestin[4], req.ethdestin[5]);
            printf("rpl.ethdestin = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X\n",
                   rpl.ethdestin[0], rpl.ethdestin[1],
                   rpl.ethdestin[2], rpl.ethdestin[3],
                   rpl.ethdestin[4], rpl.ethdestin[5]);

            printf("req.ethsource = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X ",
                   req.ethsource[0], req.ethsource[1],
                   req.ethsource[2], req.ethsource[3],
                   req.ethsource[4], req.ethsource[5]);
            printf("rpl.ethsource = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X\n",
                   rpl.ethsource[0], rpl.ethsource[1],
                   rpl.ethsource[2], rpl.ethsource[3],
                   rpl.ethsource[4], rpl.ethsource[5]);

            printf("req.ethproto  = 0x%-16.4X", ntohs(req.ethproto));
            printf("rpl.ethproto  = 0x%.4X\n", ntohs(rpl.ethproto));

            printf("req.hw        = %-18u", ntohs(req.hw));
            printf("rpl.hw        = %u\n", ntohs(rpl.hw));

            printf("req.prot      = 0x%-16.4X", ntohs(req.prot));
            printf("rpl.prot      = 0x%.4X\n", ntohs(rpl.prot));

            printf("req.hwalen    = %-18u", req.hwalen);
            printf("rpl.hwalen    = %u\n", rpl.hwalen);

            printf("req.pralen    = %-18u", req.pralen);
            printf("rpl.pralen    = %u\n", rpl.pralen);

            printf("req.opcode    = %-18u", ntohs(req.opcode));
            printf("rpl.opcode    = %u\n", ntohs(rpl.opcode));

            printf("req.shwaddr   = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X ",
                   req.shwaddr[0], req.shwaddr[1],
                   req.shwaddr[2], req.shwaddr[3],
                   req.shwaddr[4], req.shwaddr[5]);
            printf("rpl.shwaddr   = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X\n",
                   rpl.shwaddr[0], rpl.shwaddr[1],
                   rpl.shwaddr[2], rpl.shwaddr[3],
                   rpl.shwaddr[4], rpl.shwaddr[5]);

            printf("req.sprotaddr = 0x%-16.8lX", ntohl(req.sprotaddr));
            printf("rpl.sprotaddr = 0x%.8lX\n", ntohl(rpl.sprotaddr));

            printf("req.thwaddr   = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X ",
                   req.thwaddr[0], req.thwaddr[1],
                   req.thwaddr[2], req.thwaddr[3],
                   req.thwaddr[4], req.thwaddr[5]);
            printf("rpl.thwaddr   = %.2X:%.2X:%.2X:%.2X:%.2X:%.2X\n",
                   rpl.thwaddr[0], rpl.thwaddr[1],
                   rpl.thwaddr[2], rpl.thwaddr[3],
                   rpl.thwaddr[4], rpl.thwaddr[5]);

            printf("req.tprotaddr = 0x%-16.8lX", ntohl(req.tprotaddr));
            printf("rpl.tprotaddr = 0x%.8lX\n\n", ntohl(rpl.tprotaddr));
        }
    }
#endif  /* DEBUGARP */
    return 0;
}

static uint8 valid_pi_vects[] = {
    0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x68, 0x69,
    0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x78, 0x79, 0x79,
    0x7A, 0x7B, 0x7C, 0x7D, 0x7E
};

static uint8 pktsig[]      = "PKT DRVR";
static uint8 driver_name[] = "PPPD220F";

/*
 * This routine checks for PPPD packet driver presence in memory. If you
 * pass in 0, it does an automatic search and returns the first one found
 * in the allowed range. If you pass a specific number, it will check if
 * there is a PPPD packet driver there.
 */
int find_packet_driver(int pinum)
{
    uint8 *pvalid, *pvalidend;
    uint8 far *psig;
    union REGS regs;
    struct SREGS sregs;

    /* they look for a specific vector, checkit */
    if ( pinum ) {
        /* see if it is one of the valid vector numbers for packet drivers */
        if ( (pvalid = memchr(valid_pi_vects,
                              pinum,
                              sizeof(valid_pi_vects))) == NULL )
            /* no, return error indication */
            return 0;

        /* limit the following scan to one vector */
        pvalidend = pvalid + 1;
    }
    /* they are looking for the first one,
       init the following scan acordling */
    else {
        pvalid = valid_pi_vects;
        pvalidend = ((uint8 *)valid_pi_vects + sizeof(valid_pi_vects));
    }

    /* do a packet driver search */
    for ( ; pvalid < pvalidend ; pvalid++ ) {
        /* check if there is an installed packet driver there */
        if ( (psig = (uint8 far *)getvect(*pvalid)) != NULL &&
             _fmemcmp(psig + 3, pktsig, sizeof(pktsig) - 1) == 0 ) {
            /* now verify packet driver is PPPD */
            segread(&sregs);
            regs.x.bx = 0; /* older drivers expects a handle here */
            regs.h.ah = DRIVER_INFO;
            regs.h.al = 0xff;
            int86x(*pvalid, &regs, &regs, &sregs);

            /* if CY isn't set, check for PPPD driver name */
            if ( ! regs.x.cflag &&
                 _fmemcmp(driver_name,
                          MK_FP(sregs.ds, regs.x.si),
                          sizeof(driver_name) - 1) == 0 ) {

                /* found PPPD driver, try to get asy funcs addresses */
                segread(&sregs);
                regs.x.bx = sizeof(ASY_HOOKS);
                regs.h.ah = GET_ASYFUNCS;
                int86x(*pvalid, &regs, &regs, &sregs);

                /* if CY isn't set, we succeed */
                if ( ! regs.x.cflag ) {
                    asyfuncs = *(ASY_HOOKS far *)MK_FP(sregs.ds, regs.x.si);

                    return *pvalid;
                }
            }
        }
    }

    /* no packet driver was found, return error indication */
    return 0;
}

